/*
 * Copyright (c) 2015-2019, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 *  ======== main.c ========
 */

/* For usleep() */
#include <unistd.h>
#include <stdint.h>
#include <stddef.h>

/* Driver Header files */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/I2C.h>
#include <ti/drivers/SPI.h>
#include <ti/display/Display.h>
//#include <ti/drivers/Watchdog.h>

/* Driver configuration */
#include "ti_drivers_config.h"

// Peripheral drivers
#include "PCF8523_RTCC.h"

static uint8_t slaveAddress;
static Display_Handle display;

/*
 *  ======== mainThread ========
 */
void *mainThread(void *arg0)
{
    /* 1 second delay */
    uint32_t time = 1;

    uint8_t I2C_txBuffer[21];
	uint8_t I2C_rxBuffer[20];

	I2C_Handle i2c;
	I2C_Params i2cParams;
	I2C_Transaction i2cTransaction;

    /* Call driver init functions */
    GPIO_init();
    I2C_init();
    //SPI_init();
    // Watchdog_init();

    /* Open the UART display for output */
	display = Display_open(Display_Type_UART, NULL);
	if (display == NULL)
	{
		while (1)
		{
			sleep(time);
			GPIO_toggle(CONFIG_LED_GREEN);
		}
	}

    /* Configure the LED pin */
    GPIO_setConfig(CONFIG_LED_GREEN, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_LOW);
    GPIO_setConfig(CONFIG_LED_BUTTON, GPIO_CFG_OUT_STD | GPIO_CFG_OUT_LOW);

    /* Turn off user LED */
    GPIO_write(CONFIG_LED_GREEN, CONFIG_GPIO_LED_OFF);

    Display_printf(display, 0, 0, "Starting i2c.\n");

    /* Create I2C for usage */
	I2C_Params_init(&i2cParams);
	i2cParams.bitRate = I2C_400kHz;
	i2c               = I2C_open(CONFIG_I2C_RTCC, &i2cParams);
	if (i2c == NULL)
	{
		Display_printf(display, 0, 0, "Error Initializing I2C\n");
		while (1)
		{
			sleep(time);
			GPIO_toggle(CONFIG_LED_GREEN);
		}
	}
	else
	{
		Display_printf(display, 0, 0, "I2C Initialized!\n");
		GPIO_write(CONFIG_LED_BUTTON, CONFIG_GPIO_LED_ON);
		sleep(time);
		GPIO_write(CONFIG_LED_BUTTON, CONFIG_GPIO_LED_OFF);
	}

	/* Common I2C transaction setup */
	i2cTransaction.writeBuf   = I2C_txBuffer;
	i2cTransaction.writeCount = 1;
	i2cTransaction.readBuf    = I2C_rxBuffer;
	i2cTransaction.readCount  = 0;

	i2cTransaction.slaveAddress = PCF8523_I2C_ADDRESS;
	I2C_txBuffer[0]             = 0x00;

	if (I2C_transfer(i2c, &i2cTransaction))
	{
		slaveAddress = PCF8523_I2C_ADDRESS;
		Display_printf(display, 0, 0, "Detected RTCC with slave address 0x%x", PCF8523_I2C_ADDRESS);
	}
	else
	{
		i2cErrorHandler(&i2cTransaction, display);
	}

	if (slaveAddress == 0)
	{
		Display_printf(display, 0, 0, "Failed to detect the RTCC!");
		I2C_close(i2c);
		while (1)
		{
			sleep(time);
			GPIO_toggle(CONFIG_LED_GREEN);
		}
	}

	// send RTCC config
	I2C_txBuffer[0]				= 0x00;	// start address = 0x00
	I2C_txBuffer[1]				= 0x00;
	I2C_txBuffer[2]				= 0x00;
	I2C_txBuffer[3]				= 0xE0;
	I2C_txBuffer[4]				= 0x33;	// 33 seconds
	I2C_txBuffer[5]				= 0x09;	//	9 minutes
	I2C_txBuffer[6]				= 0x21;	// 21 hours
	I2C_txBuffer[7]				= 0x03;	// 3 (day: 1...31)
	I2C_txBuffer[8]				= 0x02;	// Wednesday
	I2C_txBuffer[9]				= 0x11;	// November
	I2C_txBuffer[10]			= 0x16; // xx22
	I2C_txBuffer[11]			= 0x00;	// minute alarm disabled
	I2C_txBuffer[12]			= 0x00;	// hour alarm disabled
	I2C_txBuffer[13]			= 0x00; // day alarm disabled
	I2C_txBuffer[14]			= 0x00;	// weekday alarm disabled
	I2C_txBuffer[15]			= 0x00;	// offset = 0
	I2C_txBuffer[16]			= 0x00;	// timer clkout OFF
	I2C_txBuffer[17]			= 0x07;	// TMR_A_FREQ 1/3600 Hz
	I2C_txBuffer[18]			= 0x00;	// TMR_A_REG 0
	I2C_txBuffer[19]			= 0x07;	// TMR_B_FREQ 1/3600 Hz
	I2C_txBuffer[20]			= 0x00;	// TMR_B_REG 0
	i2cTransaction.writeCount	= 21;
	i2cTransaction.readCount	= 0;

	if (I2C_transfer(i2c, &i2cTransaction))
	{
		Display_printf(display, 0, 0, "Default configuration written\n");
	}
	else
	{
		i2cErrorHandler(&i2cTransaction, display);
	}

	i2cTransaction.writeCount	= 1;
	i2cTransaction.readCount	= 20;

	if (I2C_transfer(i2c, &i2cTransaction))
	{
		Display_printf(display, 0, 0, "Register readout complete\n");
	}
	else
	{
		i2cErrorHandler(&i2cTransaction, display);
	}

	I2C_close(i2c);
	Display_printf(display, 0, 0, "I2C closed!");
    while (1)
    {
        //sleep(time);
        //GPIO_toggle(CONFIG_LED_GREEN);
        GPIO_write(CONFIG_LED_GREEN, CONFIG_GPIO_LED_ON);
    }
}
